"""Provides the base class for a widget that displays keys."""

from __future__ import annotations

from dataclasses import dataclass
from functools import partial
from typing import cast

from rich.emoji import Emoji
from textual.binding import Binding
from textual.events import Key
from textual.message import Message
from textual.widgets import OptionList
from textual.widgets.option_list import Option, OptionDoesNotExist
from typing_extensions import Final, Self

from ..dialogs import Annotation


class TestableKey(Option):
    """A class for holding details of a testable key."""

    NOTE_ICON: Final[str] = Emoji.replace(":paperclip:")
    """The icon to use to indicate a key has a note attached."""

    def __init__(self, key: str | Key, notes: str = "") -> None:
        """Initialise the testable key.

        Args:
            key: The name of the key.
        """
        key = key if isinstance(key, str) else key.key
        super().__init__(key, id=key)
        self._notes = notes
        """Holds user-entered notes about the key."""

    @property
    def notes(self) -> str:
        """The notes for the key."""
        return self._notes

    @notes.setter
    def notes(self, notes: str) -> None:
        """Set the notes for the key."""
        self._notes = notes
        assert self.id is not None
        self.set_prompt(f"{self.id} {self.NOTE_ICON}" if notes.strip() else self.id)

    def to_json(self) -> dict[str, str]:
        """Get the key data as a json-friendly data.

        Returns:
            The key data as json-friendly data.
        """
        assert self.id is not None
        return {"key": self.id, "notes": self.notes}

    @classmethod
    def from_json(cls, data: dict[str, str]) -> TestableKey:
        """Get a new `TestableKey` instance from json-friendly data.

        Args:
            data: The data to make the instance from.

        Returns:
            A fresh instance of `TestableKey`.
        """
        key = cls(data["key"])
        key.notes = data.get("notes", "")
        return key


class KeysDisplay(OptionList):
    """Base class for the key displays"""

    DEFAULT_CSS = """
    KeysDisplay {
        border: panel cornflowerblue 50%;
        width: 1fr;
        height: 1fr;
    }

    KeysDisplay:focus {
        border: panel cornflowerblue;
    }
    """

    BINDINGS = [Binding("enter", "annotate", "Annotate Key")]

    @dataclass
    class NotesUpdated(Message):
        """Message posted when the notes of a key are modified."""

        keys_list: KeysDisplay
        """The list that handled the update."""
        key: TestableKey
        """The key that was updated."""

        @property
        def control(self) -> KeysDisplay:
            """The control sending the message."""
            return self.keys_list

    def __contains__(self, key: str | Key) -> bool:
        """Is the given key in this list?

        Args:
            key: The key to test for.
        """
        try:
            _ = self.get_option(key if isinstance(key, str) else key.key)
        except OptionDoesNotExist:
            return False
        return True

    def to_json(self) -> list[dict[str, str]]:
        """Get the keys in the list as json-friendly data.

        Returns:
            A json-friendly version of the data in the list.
        """
        return [key.to_json() for key in self._contents if isinstance(key, TestableKey)]

    def from_json(self, data: list[dict[str, str]]) -> None:
        """Set the content of the list to the content of the given json-friendly data.

        Args:
            data: The data to load into the list.
        """
        self.clear_options().add_options(
            TestableKey.from_json(key) for key in data
        ).refresh_count()

    def _update_notes(self, key: TestableKey, notes: str) -> None:
        """Update the notes for the given key.

        Args:
            key: The key to update.
            notes: The notes to update the key with.
        """
        if key.notes != notes:
            self.post_message(self.NotesUpdated(self, key))
            key.notes = notes
            self._refresh_content_tracking(force=True)
            self.refresh()

    def action_annotate(self) -> None:
        """Annotate the current key."""
        if self.highlighted is not None:
            key = self.get_option_at_index(self.highlighted)
            assert isinstance(key, TestableKey) and key.id is not None
            self.app.push_screen(
                Annotation(key.id, key.notes), callback=partial(self._update_notes, key)
            )

    def get_option(self, option_id: str) -> TestableKey:
        """Override to cast the return type to the `Option` subclass.

        Args:
            option_id: The ID of the option to get.

        Returns:
            The testable key with that ID.
        """
        return cast(TestableKey, super().get_option(option_id))

    def on_mount(self) -> None:
        """Configure the list once the DOM is ready."""
        self.refresh_count()

    def refresh_count(self) -> Self:
        """Refresh the count of keys in the list as shown in the border."""
        if self.BORDER_TITLE:
            self.border_title = f"{self.BORDER_TITLE} ({self.option_count})"
        return self



"""A list for showing expected keys."""

from textual.keys import KEY_TO_UNICODE_NAME, Keys

from .keys_display import KeysDisplay, TestableKey


class ExpectedKeys(KeysDisplay):
    """A widget to show the keys we expect to encounter."""

    BORDER_TITLE = "Expected Keys"

    def __init__(self) -> None:
        """Initialise the widget."""
        super().__init__(
            *[
                TestableKey(key)
                for key in sorted(
                    set(key.value for key in Keys) | set(KEY_TO_UNICODE_NAME.keys())
                )
                if not key.startswith("<")
            ],
        )



"""Notepad widget for the general session notes."""

from textual.events import Key
from textual.widgets import TextArea


class Notepad(TextArea):
    """A notepad widget for the key recording session data."""

    BORDER_TITLE = "Notepad"

    DEFAULT_CSS = """
    Notepad {
        border: panel cornflowerblue 50%;
        height: 1fr;
    }

    Notepad:focus {
        border: panel cornflowerblue;
    }
    """

    def on_mount(self) -> None:
        """Configure the notepad once mounted in the DOM."""
        self.show_line_numbers = False

    async def _on_key(self, event: Key) -> None:
        """Allow tab to move along the focus chain."""
        if event.key == "tab":
            event.prevent_default()
            self.screen.focus_next()



"""A list for showing unexpected keys."""

from .keys_display import KeysDisplay


class UnexpectedKeys(KeysDisplay):
    """A widget to show the keys we didn't expect to encounter."""

    BORDER_TITLE = "Unexpected Keys"



"""Provides a dialog for editing an annotation."""

from textual import on
from textual.app import ComposeResult
from textual.binding import Binding
from textual.containers import Horizontal, Vertical
from textual.events import Key
from textual.screen import ModalScreen
from textual.widgets import Button, TextArea


class DialogTextArea(TextArea):
    """A version of `TextArea` that's friendly to this dialog."""

    async def _on_key(self, event: Key) -> None:
        """Allow tab to move along the focus chain."""
        if event.key == "tab":
            event.prevent_default()
            self.screen.focus_next()


class Annotation(ModalScreen[str]):
    """A dialog for editing an annotation."""

    DEFAULT_CSS = """
    Annotation {
        align: center middle;
    }

    Vertical {
        width: 50%;
        height: 50%;
        border: panel cornflowerblue;
        padding: 1 2;
        background: $panel;
    }

    Horizontal {
        align: right middle;
        height: auto;
        margin-top: 1;
    }

    Button {
        margin-left: 1;
    }
    """

    BINDINGS = [
        Binding("escape", "app.pop_screen", show=False),
    ]

    def __init__(self, key: str, annotation: str = "") -> None:
        """Initialise the annotation dialog.

        Args:
            annotation: The note to edit.
        """
        super().__init__()
        self._key = key
        self._annotation = annotation

    def compose(self) -> ComposeResult:
        with Vertical() as panel:
            panel.border_title = f"Annotation for {self._key}"
            yield DialogTextArea(self._annotation)
            with Horizontal():
                yield Button("Save", id="save")
                yield Button("Cancel", id="cancel")

    def on_mount(self) -> None:
        """Configure the dialog once mounted."""
        self.query_one(TextArea).show_line_numbers = False

    @on(Button.Pressed, "#save")
    def save(self) -> None:
        """Handle the save button."""
        self.dismiss(self.query_one(TextArea).text)

    @on(Button.Pressed, "#cancel")
    def cancel(self) -> None:
        """Handle the cancel button."""
        self.app.pop_screen()



"""A widget for holding and displaying environmental information."""

from __future__ import annotations

import os
import platform

from textual import __version__
from textual.widgets import DataTable


def _guess_term() -> str:
    """Try and guess which terminal is being used.

    Returns:
        The best guess at the name of the terminal.
    """

    # Look in a couple of generic locations for the name of the terminal.
    term_program = os.environ.get("TERM_PROGRAM") or os.environ.get("TERMINAL_NAME")

    if term_program is None:
        # Seems we couldn't get it that way. Let's check for some of the
        # more common terminal signatures.
        if "ALACRITTY_WINDOW_ID" in os.environ:
            term_program = "Alacritty"
        elif "KITTY_PID" in os.environ:
            term_program = "Kitty"
        elif "WT_SESSION" in os.environ:
            term_program = "Windows Terminal"
        elif "INSIDE_EMACS" in os.environ and os.environ["INSIDE_EMACS"]:
            term_program = (
                f"GNU Emacs {' '.join(os.environ['INSIDE_EMACS'].split(','))}"
            )
        elif "JEDITERM_SOURCE_ARGS" in os.environ:
            term_program = "PyCharm"
        elif "GNOME_TERMINAL_SCREEN" in os.environ:
            term_program = "GNOME Terminal"
        elif "XTERM_VERSION" in os.environ:
            term_program = os.environ.get("XTERM_VERSION") or "XTerm"
        elif "TERMINATOR_UUID" in os.environ:
            term_program = "Terminator"

    else:
        # See if we can pull out some sort of version information too.
        term_version = os.environ.get("TERM_PROGRAM_VERSION") or os.environ.get(
            "TERMINAL_VERSION_STRING"
        )
        if term_version is not None:
            term_program = f"{term_program} ({term_version})"

    # Seems we can't work this out.
    if term_program is None:
        term_program = "*Unknown*"

    # Check for running under screen. As you look at this you might think
    # "what about tmux too?" -- good point; but we'll be picking up tmux as
    # the terminal type, because of how it takes over TERM_PROGRAM.
    if "STY" in os.environ:
        term_program += " (inside screen)"

    return term_program


class Environment(DataTable):
    """Widget that holds and shows the test environment details."""

    BORDER_TITLE = "Environment"

    DEFAULT_CSS = """
    Environment {
        height: auto;
        border: panel cornflowerblue 50%;
        color: $text 50%;
        scrollbar-gutter: stable;
    }

    Environment > .datatable--header {
        color: $text 50%;
    }

    Environment:focus {
        border: panel cornflowerblue;
        color: $text;
    }

    Environment:focus > .datatable--header {
        color: $text;
    }
    """

    def on_mount(self) -> None:
        """Configure the environment display once the DOM is ready."""
        self.cursor_type = "none"
        self.add_column("Name", key="name")
        self.add_column("Value", key="value")
        self.add_row("OS System", platform.system())
        self.add_row("OS Release", platform.release())
        self.add_row("OS Version", platform.version())
        self.add_row("Textual", __version__)
        self.add_row("Terminal", _guess_term())

    def to_json(self) -> dict[str, str]:
        """Get the environment information as json-friendly data.

        Returns:
            The environment data as json-friendly data.
        """
        return {
            "os_system": platform.system(),
            "os_release": platform.release(),
            "os_version": platform.version(),
            "textual": __version__,
            "terminal": _guess_term(),
        }

    def is_similar(self, compare: dict[str, str]) -> bool:
        """Is the current environment the similar to another environment?

        Args:
            compare: The environment to compare to the current one.

        Returns:
            `True` if they're similar, `False` if different.
        """
        current = self.to_json()
        return all(current[key] == compare[key] for key in ("os_system", "terminal"))



"""The main application class."""

from textual.app import App

from .screens import Main

App.BINDINGS = []  # Nuke the default Textual bindings.


class TextualKeyRecorder(App[None]):
    """Application class for the Textual key recorder."""

    TITLE = "Textual Key Recorder"

    def on_mount(self) -> None:
        """Set up the application once the DOM is ready."""
        self.push_screen(Main())


def run() -> None:
    """Run the application."""
    TextualKeyRecorder().run()



"""A list for showing triggered keys."""

from .keys_display import KeysDisplay


class TriggeredKeys(KeysDisplay):
    """A widget to show the keys that have been triggered."""

    BORDER_TITLE = "Triggered Keys"



"""A widget for grabbing key input data."""

from dataclasses import dataclass

from textual._xterm_parser import XTermParser
from textual.events import Key
from textual.message import Message
from textual.widgets import Static


class KeyInput(Static, can_focus=True):
    """The widget where a key should be input."""

    BORDER_TITLE = "Key Press"
    BORDER_SUBTITLE = "Focus this widget to test a key"

    DEFAULT_CSS = """
    KeyInput {
        border: panel cornflowerblue 50%;
        height: 5;
        padding: 1;
        text-align: center;
        width: 1fr;
    }

    KeyInput:focus {
        border: panel cornflowerblue;
    }
    """

    @dataclass
    class Triggered(Message):
        """Event raised when a key is triggered."""

        key: Key
        """The key that was triggered."""

    @dataclass
    class Unknown(Message):
        """Event raised when an unknown sequence comes in."""

        sequence: str
        """The sequence that came in."""

    def __init__(self) -> None:
        """Initialise the widget."""
        super().__init__("Press a key to test...")
        self.tab_tested = False
        self.shift_tab_tested = False

    def _unknown_sequence(self, sequence: str) -> None:
        """Report an unknown sequence.

        Args:
            sequence: The unknown sequence to report.
        """
        self.post_message(self.Unknown(repr(sequence)[1:-1]))

    def on_focus(self) -> None:
        """Hook into the unknown sequence code while we have focus.."""
        XTermParser._reissued_sequence_debug_book = self._unknown_sequence

    def on_blur(self) -> None:
        """Unhook from the unknown sequence capture when focus is lost."""
        XTermParser._reissued_sequence_debug_book = None

    def on_key(self, event: Key):
        """Handle keystrokes.

        Args:
            event: The key event to handle.
        """
        # Just let focus switching do its thing if we've already triggered it.
        if (event.key == "tab" and self.tab_tested) or (
            event.key == "shift+tab" and self.shift_tab_tested
        ):
            return

        self.update(f"{event!r}")
        self.post_message(self.Triggered(event))

        # If this is our first attempt at a focus switch don't switch but
        # record it's fine form now on.
        if event.key == "tab":
            self.tab_tested = True
            event.stop()
        elif event.key == "shift+tab":
            self.shift_tab_tested = True
            event.stop()



"""The main screen for the application."""

from __future__ import annotations

from json import JSONDecodeError, dumps, loads
from pathlib import Path

from textual import on
from textual.app import ComposeResult
from textual.binding import Binding
from textual.containers import Horizontal, Vertical
from textual.events import Key
from textual.message import Message
from textual.reactive import var
from textual.screen import Screen
from textual.widgets import Button, Footer, Header
from textual_fspicker import FileOpen, FileSave, Filters

from ..dialogs import YesNo
from ..widgets import (
    Environment,
    ExpectedKeys,
    KeyInput,
    KeysDisplay,
    Notepad,
    TestableKey,
    TriggeredKeys,
    UnexpectedKeys,
    UnknownKeys,
)


class AdminArea(Horizontal):
    """The area of the application where admin tasks can take place."""

    DEFAULT_CSS = """
    AdminArea {
        margin-top: 1;
        margin-bottom: 1;
    }

    Environment {
        margin-top: 1;
    }

    UnknownKeys {
        margin-top: 1;
    }

    ExpectedKeys {
        height: 3fr;
        margin-bottom: 1;
    }
    """

    BINDINGS = [
        Binding("ctrl+l", "load", "Load progress"),
        Binding("ctrl+s", "save", "Save progress"),
        Binding("ctrl+q", "quit_recorder", "Quit"),
    ]

    progress_file: var[Path | None] = var(None)
    """The file where progress is recorded."""

    dirty: var[bool] = var(False)
    """Is the data dirty?"""

    FILE_EXTENSION = ".tkrec"
    """The extension used for the files saved by this application."""

    FILTERS = Filters(
        (
            "Textual Keys Recording",
            lambda p: p.suffix.lower() == AdminArea.FILE_EXTENSION,
        )
    )
    """The filters to use for the file dialogs."""

    class DataLoaded(Message):
        """Message posted when fresh data is loaded."""

    def compose(self) -> ComposeResult:
        """Compose the child widgets."""
        with Vertical():
            yield ExpectedKeys()
            yield Notepad()
        with Vertical():
            with Horizontal():
                yield TriggeredKeys()
                with Vertical():
                    yield UnexpectedKeys()
                    yield UnknownKeys()
            yield Environment()

    def to_json(self) -> dict[str, dict[str, str] | list[dict[str, str]] | str]:
        """Get the state of the screen as a json-friendly data structure.

        Returns:
            A json-friendly structure of the data.
        """
        return {
            "environment": self.query_one(Environment).to_json(),
            "expected": self.query_one(ExpectedKeys).to_json(),
            "unexpected": self.query_one(UnexpectedKeys).to_json(),
            "unknown": self.query_one(UnknownKeys).to_json(),
            "triggered": self.query_one(TriggeredKeys).to_json(),
            "notes": self.query_one(Notepad).text,
        }

    def _save_data(self, save_file: Path | None) -> None:
        """Save the current state to the given file.

        Args:
            save_file: The file to save to, or `None` if no save should
                happen.
        """
        if save_file is not None:
            save_file = save_file.with_suffix(self.FILE_EXTENSION)
            self.progress_file = save_file
            save_file.write_text(dumps(self.to_json(), indent=4))
            self.dirty = False
            self.notify(str(save_file), title="Saved")

    def action_save(self) -> None:
        """Save the current progress."""
        if self.progress_file is None:
            self.app.push_screen(
                FileSave(filters=self.FILTERS), callback=self._save_data
            )
        else:
            self._save_data(self.progress_file)

    def _load_data(self, load_file: Path | None) -> None:
        """Load state from the given file.

        Args:
            load_file: The file to load from, or `None` if no load is to
                happen.
        """
        if load_file is not None:

            def error():
                self.notify(
                    "Not a valid key recording session file.",
                    title="Error loading file",
                    severity="error",
                    timeout=6,
                )

            try:
                data = loads(load_file.read_text())
            except JSONDecodeError:
                error()
                return
            if all(
                required in data
                for required in ("expected", "unexpected", "triggered", "unknown")
            ):
                self.progress_file = load_file
                self.query_one(ExpectedKeys).from_json(data["expected"])
                self.query_one(UnexpectedKeys).from_json(data["unexpected"])
                self.query_one(TriggeredKeys).from_json(data["triggered"])
                self.query_one(UnknownKeys).from_json(data["unknown"])
                self.query_one(Notepad).load_text(data.get("notes", ""))
                self.post_message(self.DataLoaded())
                if not self.query_one(Environment).is_similar(
                    data.get("environment", {})
                ):
                    self.notify(
                        "Data loaded for a dissimilar environment!",
                        severity="warning",
                        timeout=8,
                    )
                self.dirty = False
            else:
                error()

    def _initiate_load(self, really_load: bool = True) -> None:
        """Core file loading method.

        Args:
            really_load: Flag to say if we should really load the file.
        """
        if really_load:
            self.app.push_screen(
                FileOpen(filters=self.FILTERS), callback=self._load_data
            )

    def action_load(self) -> None:
        """Start the process of loading a progress file."""
        if self.dirty:
            self.app.push_screen(
                YesNo("You have unsaved data, do you really want to load?"),
                callback=self._initiate_load,
            )
        else:
            self._initiate_load()

    def _really_quit(self, confirmed: bool) -> None:
        """Unconditionally exit the application.

        Args:
            confirmed: Was the quit confirmed by the user?
        """
        if confirmed:
            self.app.exit()

    def action_quit_recorder(self) -> None:
        """Quit the recorder."""
        if self.dirty:
            self.app.push_screen(
                YesNo("You have unsaved data, do you really want to quit?"),
                callback=self._really_quit,
            )
        else:
            self.app.exit()

    def _refresh_subtitle(self) -> None:
        """Refresh the subtitle of the app."""
        self.app.sub_title = (
            "Untitled" if self.progress_file is None else str(self.progress_file)
        ) + (" (unsaved)" if self.dirty else "")

    def _watch_progress_file(self) -> None:
        """Refresh when the progress file name changes."""
        self._refresh_subtitle()

    def _watch_dirty(self) -> None:
        """Refresh when the dirty flag changes."""
        self._refresh_subtitle()


class Toolbar(Vertical):
    """Toolbar widget."""

    DEFAULT_CSS = """
    Toolbar {
        border: round cornflowerblue 50%;
        background: $panel;
        width: auto;
        height: auto;
    }

    Toolbar Button, Toolbar Button:hover, Toolbar Button.-active {
        border: none;
        height: auto;
    }
    """

    BINDINGS = [
        Binding("up", "focus_previous"),
        Binding("down", "focus_next"),
    ]


class Main(Screen):
    """The main screen of the application."""

    CSS = """
    #inputs {
        height: auto;
    }
    """

    def compose(self) -> ComposeResult:
        """Compose the child widgets."""
        yield Header()
        with Horizontal(id="inputs"):
            yield KeyInput()
            with Toolbar():
                yield Button("Load", id="load")
                yield Button("Save", id="save")
                yield Button("Quit", id="quit_recorder")
        yield AdminArea()
        yield Footer()

    @on(KeysDisplay.NotesUpdated)
    @on(Notepad.Changed)
    def _mark_dirty(self) -> None:
        """Mark the data as dirty in response to events."""
        self.query_one(AdminArea).dirty = True

    @on(AdminArea.DataLoaded)
    def _reset_recorder(self) -> None:
        """Configure things after new data has been loaded."""
        triggered = self.query_one(TriggeredKeys)
        self.query_one(KeyInput).tab_tested = "tab" in triggered
        self.query_one(KeyInput).shift_tab_tested = "shift+tab" in triggered

    def _add(self, key: str | Key | TestableKey, target: KeysDisplay) -> None:
        """Add a key to the target list.

        Args:
            key: The key to record.
            target: The list to record the key in.
        """
        target.add_option(
            key if isinstance(key, TestableKey) else TestableKey(key)
        ).refresh_count().action_last()
        self._mark_dirty()

    @on(KeyInput.Triggered)
    def mark_key_as_triggered(self, event: KeyInput.Triggered) -> None:
        """Handle a key being triggered.

        Args:
            event: The key trigger event.
        """

        # Pull out the various lists.
        expected = self.query_one(ExpectedKeys)
        unexpected = self.query_one(UnexpectedKeys)
        triggered = self.query_one(TriggeredKeys)

        # First off, get the key that was triggered.
        triggered_key = event.key

        # Figure out where the key should land, if at all.
        if triggered_key in expected:
            # It's in the expected list, so move it over to the triggered list.
            self._add(expected.get_option(triggered_key.key), triggered)
            expected.remove_option(triggered_key.key).refresh_count()
        elif triggered_key.is_printable:
            # Printable keys that we weren't expecting are treated as kind
            # of expected, so if it isn't in the triggered list yet, add it
            # now.
            if triggered_key not in triggered:
                self._add(triggered_key, triggered)
        elif triggered_key not in triggered and triggered_key not in unexpected:
            # It's not expected and hasn't been triggered, and so far hasn't
            # landed in the unexpected list; so add it now.
            self._add(triggered_key, unexpected)

    @on(KeyInput.Unknown)
    def record_unknown_key(self, event: KeyInput.Unknown) -> None:
        """Record an unknown key.

        Args:
            event: The unknown key event.
        """
        unknown = self.query_one(UnknownKeys)
        if event.sequence not in unknown:
            self._add(event.sequence, unknown)
        self.app.bell()
        self.notify(
            event.sequence,
            title="Unknown sequence received",
            severity="warning",
            timeout=10,
        )

    @on(Button.Pressed)
    async def toolbar_button(self, event: Button.Pressed) -> None:
        """Handle toolbar buttons.

        Args:
            event: The button press event.
        """
        if event.control.id and isinstance(event.control.parent, Toolbar):
            await self.query_one(AdminArea).run_action(event.control.id)



"""Provides a simple yes/no dialog."""

from textual import on
from textual.app import ComposeResult
from textual.binding import Binding
from textual.containers import Horizontal, Vertical
from textual.screen import ModalScreen
from textual.widgets import Button, Label


class YesNo(ModalScreen[bool]):
    """A simple yes/no dialog."""

    DEFAULT_CSS = """
    YesNo {
        align: center middle;
    }

    Vertical {
        width: auto;
        height: auto;
        border: thick $accent;
        padding: 1 2;
        background: $panel;
    }

    Horizontal {
        align: right middle;
        width: auto;
        height: auto;
        margin-top: 1;
    }

    Button {
        margin-left: 1;
    }
    """

    BINDINGS = [
        Binding("escape", "app.pop_screen", show=False),
    ]

    def __init__(self, question: str = "") -> None:
        """Initialise the question dialog.

        Args:
            question: The question to ask.
        """
        super().__init__()
        self._question = question

    def compose(self) -> ComposeResult:
        """Compose the content of the dialog."""
        with Vertical():
            yield Label(self._question)
            with Horizontal():
                yield Button("Yes", id="yes")
                yield Button("No", id="no")

    @on(Button.Pressed)
    def decision(self, event: Button.Pressed) -> None:
        """Handle the user's decision.

        Args:
            event: The button press event that is the decision.
        """
        self.dismiss(event.button.id == "yes")



"""Widgets for use in the application."""

from .environment import Environment
from .expected_keys import ExpectedKeys
from .key_input import KeyInput
from .keys_display import KeysDisplay, TestableKey
from .notepad import Notepad
from .triggered_keys import TriggeredKeys
from .unexpected_keys import UnexpectedKeys
from .unknown_keys import UnknownKeys

__all__ = [
    "Environment",
    "ExpectedKeys",
    "KeyInput",
    "KeysDisplay",
    "Notepad",
    "TriggeredKeys",
    "UnexpectedKeys",
    "UnknownKeys",
    "TestableKey",
]



"""A list for showing unknown key sequences."""

from .keys_display import KeysDisplay


class UnknownKeys(KeysDisplay):
    """A widget to show unknown key sequences."""

    BORDER_TITLE = "Unknown Sequences"
